@@ -80,22 +80,6 @@ private:
      */
     void processBasicWallsSkinInfill(SliceDataStorage& storage, const size_t mesh_order_idx, const std::vector<size_t>& mesh_order, ProgressStageEstimator& inset_skin_progress_estimate);
 
-    /*!
-     * Generate areas for the gaps between outer wall and the outline where the first wall doesn't fit.
-     * These areas should be filled with a skin-like pattern, so that these skin lines get combined into one line with gradual changing width.
-     * 
-     * \param[in,out] storage fetches the SliceLayerPart::insets and SliceLayerPart::outline and generates the outline_gaps in SliceLayerPart
-     */
-    void processOutlineGaps(SliceDataStorage& storage);
-
-    /*!
-     * Generate areas for the gaps between walls where the next inset doesn't fit.
-     * These areas should be filled with a skin-like pattern, so that these skin lines get combined into one line with gradual changing width.
-     * 
-     * \param[in,out] storage fetches the perimeter information (see SliceLayerPart::insets and SkinPart::insets) and generates the other perimeter_gaps in SliceLayerPart and SkinPart
-     */
-    void processPerimeterGaps(SliceDataStorage& storage);
-
     /*!
      * Process the mesh to be an infill mesh: limit all outlines to within the infill of normal meshes and subtract their volume from the infill of those meshes
      * 
@@ -144,7 +128,7 @@ private:
      * \brief Generate the inset polygons which form the walls.
      * \param layer_nr The layer for which to generate the insets.
      */
-    void processInsets(SliceMeshStorage& mesh, size_t layer_nr);
+    void processWalls(SliceMeshStorage& mesh, size_t layer_nr);
 
     /*!
      * Generate the outline of the ooze shield.
