@@ -346,6 +346,11 @@ void GCodeExport::setZ(int z)
     current_layer_z = z;
 }
 
+void GCodeExport::addExtraPrimeAmount(double extra_prime_volume)
+{
+    extruder_attr[current_extruder].prime_volume += extra_prime_volume;
+}
+
 void GCodeExport::setFlowRateExtrusionSettings(double max_extrusion_offset, double extrusion_offset_factor)
 {
     this->max_extrusion_offset = max_extrusion_offset;
@@ -1167,7 +1172,8 @@ void GCodeExport::writeFanCommand(double speed)
     }
     else if (speed > 0)
     {
-        *output_stream << "M106 S" << PrecisionedDouble{1, speed * 255 / 100};
+        const bool should_scale_zero_to_one = Application::getInstance().current_slice->scene.settings.get<bool>("machine_scale_fan_speed_zero_to_one");
+        *output_stream << "M106 S" << PrecisionedDouble{(should_scale_zero_to_one ? 2u : 1u), (should_scale_zero_to_one ? speed : speed * 255) / 100};
         if (fan_number)
         {
             *output_stream << " P" << fan_number;
