@@ -1,4 +1,4 @@
-//Copyright (c) 2018 Ultimaker B.V.
+//Copyright (c) 2020 Ultimaker B.V.
 //CuraEngine is released under the terms of the AGPLv3 or higher.
 
 #include "GyroidInfill.h"
@@ -14,13 +14,12 @@ GyroidInfill::GyroidInfill() {
 GyroidInfill::~GyroidInfill() {
 }
 
-void GyroidInfill::generateTotalGyroidInfill(Polygons& result_lines, bool zig_zaggify, coord_t outline_offset, coord_t infill_line_width, coord_t line_distance, const Polygons& in_outline, coord_t z)
+void GyroidInfill::generateTotalGyroidInfill(Polygons& result_lines, bool zig_zaggify, coord_t line_distance, const Polygons& in_outline, coord_t z)
 {
     // generate infill based on the gyroid equation: sin_x * cos_y + sin_y * cos_z + sin_z * cos_x = 0
     // kudos to the author of the Slic3r implementation equation code, the equation code here is based on that
 
-    const Polygons outline = in_outline.offset(outline_offset);
-    const AABB aabb(outline);
+    const AABB aabb(in_outline);
 
     int pitch = line_distance * 2.41; // this produces similar density to the "line" infill pattern
     int num_steps = 4;
@@ -71,7 +70,7 @@ void GyroidInfill::generateTotalGyroidInfill(Polygons& result_lines, bool zig_za
                 for (unsigned i = 0; i < num_coords; ++i)
                 {
                     Point current(x + ((num_columns & 1) ? odd_line_coords[i] : even_line_coords[i])/2 + pitch, y + (coord_t)(i * step));
-                    bool current_inside = outline.inside(current, true);
+                    bool current_inside = in_outline.inside(current, true);
                     if (!is_first_point)
                     {
                         if (last_inside && current_inside)
@@ -84,7 +83,8 @@ void GyroidInfill::generateTotalGyroidInfill(Polygons& result_lines, bool zig_za
                             // line hits the boundary, add the part that's inside the boundary
                             Polygons line;
                             line.addLine(last, current);
-                            line = outline.intersectionPolyLines(line);
+                            constexpr bool restitch = false; // only a single line doesn't need stitching
+                            line = in_outline.intersectionPolyLines(line, restitch);
                             if (line.size() > 0)
                             {
                                 // some of the line is inside the boundary
@@ -162,7 +162,7 @@ void GyroidInfill::generateTotalGyroidInfill(Polygons& result_lines, bool zig_za
                 for (unsigned i = 0; i < num_coords; ++i)
                 {
                     Point current(x + (coord_t)(i * step), y + ((num_rows & 1) ? odd_line_coords[i] : even_line_coords[i])/2);
-                    bool current_inside = outline.inside(current, true);
+                    bool current_inside = in_outline.inside(current, true);
                     if (!is_first_point)
                     {
                         if (last_inside && current_inside)
@@ -175,7 +175,8 @@ void GyroidInfill::generateTotalGyroidInfill(Polygons& result_lines, bool zig_za
                             // line hits the boundary, add the part that's inside the boundary
                             Polygons line;
                             line.addLine(last, current);
-                            line = outline.intersectionPolyLines(line);
+                            constexpr bool restitch = false; // only a single line doesn't need stitching
+                            line = in_outline.intersectionPolyLines(line, restitch);
                             if (line.size() > 0)
                             {
                                 // some of the line is inside the boundary
@@ -233,7 +234,7 @@ void GyroidInfill::generateTotalGyroidInfill(Polygons& result_lines, bool zig_za
 
         int chain_ends_remaining = chains[0].size() * 2;
 
-        for (ConstPolygonRef outline_poly : outline)
+        for (ConstPolygonRef outline_poly : in_outline)
         {
             std::vector<Point> connector_points; // the points that make up a connector line
 
@@ -274,7 +275,7 @@ void GyroidInfill::generateTotalGyroidInfill(Polygons& result_lines, bool zig_za
                     }
                 }
 
-                if (outline_point_index == 0 || vSize2(op0 - cur_point) > 100)
+                if (outline_point_index == 0 || vSize2(op0 - cur_point) > MM2INT(0.1))
                 {
                     // this is either the first outline point or it is another outline point that is not too close to cur_point
 
@@ -313,7 +314,7 @@ void GyroidInfill::generateTotalGyroidInfill(Polygons& result_lines, bool zig_za
                     // make the chain end the current point and add it to the connector line
                     cur_point = chains[point_index][chain_index];
 
-                    if (drawing && connector_points.size() > 0 && vSize2(cur_point - connector_points.back()) < 100)
+                    if (drawing && connector_points.size() > 0 && vSize2(cur_point - connector_points.back()) < MM2INT(0.1))
                     {
                         // this chain end will be too close to the last connector point so throw away the last connector point
                         connector_points.pop_back();
@@ -335,10 +336,7 @@ void GyroidInfill::generateTotalGyroidInfill(Polygons& result_lines, bool zig_za
 
                         if (chain_index != connector_start_chain_index && connected_to[(point_index + 1) % 2][chain_index] != connector_start_chain_index)
                         {
-                            for (unsigned pi = 1; pi < connector_points.size(); ++pi)
-                            {
-                                result.addLine(connector_points[pi - 1], connector_points[pi]);
-                            }
+                            result.add(connector_points);
                             drawing = false;
                             connector_points.clear();
                             // remember the connection
@@ -389,15 +387,9 @@ void GyroidInfill::generateTotalGyroidInfill(Polygons& result_lines, bool zig_za
             {
                 // output the connector line segments from the last chain to the first point in the outline
                 connector_points.push_back(outline_poly[0]);
-                for (unsigned pi = 1; pi < connector_points.size(); ++pi)
-                {
-                    result.addLine(connector_points[pi - 1], connector_points[pi]);
-                }
+                result.add(connector_points);
                 // output the connector line segments from the first point in the outline to the first chain
-                for (unsigned pi = 1; pi < path_to_first_chain.size(); ++pi)
-                {
-                    result.addLine(path_to_first_chain[pi - 1], path_to_first_chain[pi]);
-                }
+                result.add(path_to_first_chain);
             }
 
             if (chain_ends_remaining < 1)
