@@ -6,11 +6,12 @@
 
 #include "../utils/IntPoint.h"
 
-namespace cura 
+namespace cura
 {
 
-struct CombPath : public  std::vector<Point> //!< A single path either inside or outise the parts
+class CombPath : public  std::vector<Point> //!< A single path either inside or outise the parts
 {
+public:
     bool cross_boundary = false; //!< Whether the path crosses a boundary.
 };
 
