@@ -1,6 +1,8 @@
-//Copyright (C) 2016 Ultimaker
-//Released under terms of the AGPLv3 License
+//Copyright (c) 2018 Ultimaker B.V.
+//CuraEngine is released under the terms of the AGPLv3 or higher.
 
+#include <assert.h>
+#include "../gcodeExport.h"
 #include "NozzleTempInsert.h"
 
 namespace cura
