@@ -1,7 +1,7 @@
-//Copyright (c) 2018 Ultimaker B.V.
+//Copyright (c) 2022 Ultimaker B.V.
 //CuraEngine is released under the terms of the AGPLv3 or higher.
 
-#include "clipper.hpp"
+#include <polyclipping/clipper.hpp>
 
 #include "Application.h" //To get settings.
 #include "ExtruderTrain.h"
@@ -37,15 +37,23 @@ void Raft::generate(SliceDataStorage& storage)
                                         .difference(ooze_shield.offset(-distance - shield_line_width_layer0 / 2, ClipperLib::jtRound)); // end distance inside shield
         storage.raftOutline = storage.raftOutline.unionPolygons(ooze_shield_raft);
     }
-    const coord_t smoothing = settings.get<coord_t>("raft_smoothing");
-    storage.raftOutline = storage.raftOutline.offset(smoothing, ClipperLib::jtRound).offset(-smoothing, ClipperLib::jtRound); // remove small holes and smooth inward corners
+
+    if(settings.get<bool>("raft_remove_inside_corners"))
+    {
+        storage.raftOutline.makeConvex();
+    }
+    else
+    {
+        const coord_t smoothing = settings.get<coord_t>("raft_smoothing");
+        storage.raftOutline = storage.raftOutline.offset(smoothing, ClipperLib::jtRound).offset(-smoothing, ClipperLib::jtRound); // remove small holes and smooth inward corners
+    }
 }
 
 coord_t Raft::getTotalThickness()
 {
     const ExtruderTrain& train = Application::getInstance().current_slice->scene.current_mesh_group->settings.get<ExtruderTrain&>("adhesion_extruder_nr");
     return train.settings.get<coord_t>("raft_base_thickness")
-        + train.settings.get<coord_t>("raft_interface_thickness")
+        + train.settings.get<size_t>("raft_interface_layers") * train.settings.get<coord_t>("raft_interface_thickness")
         + train.settings.get<size_t>("raft_surface_layers") * train.settings.get<coord_t>("raft_surface_thickness");
 }
 
@@ -91,7 +99,7 @@ size_t Raft::getTotalExtraLayers()
     {
         return 0;
     }
-    return 2 + train.settings.get<size_t>("raft_surface_layers") + getFillerLayerCount();
+    return 1 + train.settings.get<size_t>("raft_interface_layers") + train.settings.get<size_t>("raft_surface_layers") + getFillerLayerCount();
 }
 
 
