@@ -22,7 +22,7 @@ struct Temperature
     /*
      * \brief Casts a double to a Temperature instance.
      */
-    Temperature(double value) : value(value) {};
+    Temperature(double value_t) : value(value_t) {};
 
     /*
      * \brief Casts the Temperature instance to a double.
@@ -62,4 +62,4 @@ struct Temperature
 
 }
 
-#endif //TEMPERATURE_H
\ No newline at end of file
+#endif //TEMPERATURE_H
