@@ -6,13 +6,15 @@
 
 
 //Include Clipper to get the ClipperLib::IntPoint definition, which we reuse as Point definition.
-#include "clipper.hpp"
+#include <polyclipping/clipper.hpp>
 
 namespace cura
 {
 
 using coord_t = ClipperLib::cInt;
 
+static inline coord_t operator "" _mu(unsigned long long i) { return i; };
+
 #define INT2MM(n) (static_cast<double>(n) / 1000.0)
 #define INT2MM2(n) (static_cast<double>(n) / 1000000.0)
 #define MM2INT(n) (static_cast<coord_t>((n) * 1000 + 0.5 * (((n) > 0) - ((n) < 0))))
