@@ -1,4 +1,4 @@
-//Copyright (c) 2018 Ultimaker B.V.
+//Copyright (c) 2020 Ultimaker B.V.
 //CuraEngine is released under the terms of the AGPLv3 or higher.
 
 #ifndef UTILS_INT_POINT_H
@@ -11,7 +11,7 @@ Integer points are used to avoid floating point rounding errors, and because Cli
 #define INLINE static inline
 
 //Include Clipper to get the ClipperLib::IntPoint definition, which we reuse as Point definition.
-#include "clipper.hpp"
+#include <polyclipping/clipper.hpp>
 #include <cmath>
 #include <functional> // for hash function object
 #include <iostream> // auto-serialization / auto-toString()
@@ -59,6 +59,8 @@ INLINE Point operator/(const Point& p0, const Point& p1) { return Point(p0.X/p1.
 INLINE Point& operator += (Point& p0, const Point& p1) { p0.X += p1.X; p0.Y += p1.Y; return p0; }
 INLINE Point& operator -= (Point& p0, const Point& p1) { p0.X -= p1.X; p0.Y -= p1.Y; return p0; }
 
+INLINE bool operator < (const Point& p0, const Point& p1) { return p0.X < p1.X || (p0.X == p1.X && p0.Y < p1.Y); }
+
 /* ***** NOTE *****
    TL;DR: DO NOT implement operators *= and /= because of the default values in ClipperLib::IntPoint's constructor.
 
@@ -93,6 +95,11 @@ INLINE bool shorterThen(const Point& p0, const coord_t len)
     return vSize2(p0) <= len * len;
 }
 
+INLINE bool shorterThan(const Point& p0, const coord_t len)
+{
+    return shorterThen(p0, len);
+}
+
 INLINE coord_t vSize(const Point& p0)
 {
     return sqrt(vSize2(p0));
@@ -130,6 +137,11 @@ INLINE coord_t dot(const Point& p0, const Point& p1)
     return p0.X * p1.X + p0.Y * p1.Y;
 }
 
+INLINE coord_t cross(const Point& p0, const Point& p1)
+{
+    return p0.X * p1.Y - p0.Y * p1.X;
+}
+
 INLINE int angle(const Point& p)
 {
     double angle = std::atan2(p.X, p.Y) / M_PI * 180.0;
@@ -137,6 +149,12 @@ INLINE int angle(const Point& p)
     return angle;
 }
 
+// Identity function, used to be able to make templated algorithms where the input is sometimes points, sometimes things that contain or can be converted to points.
+INLINE const Point& make_point(const Point& p)
+{
+    return p;
+}
+
 }//namespace cura
 
 namespace std {
@@ -189,6 +207,14 @@ public:
         matrix[3] = matrix[0];
     }
 
+    static PointMatrix scale(double s)
+    {
+        PointMatrix ret;
+        ret.matrix[0] = s;
+        ret.matrix[3] = s;
+        return ret;
+    }
+
     Point apply(const Point p) const
     {
         return Point(p.X * matrix[0] + p.Y * matrix[1], p.X * matrix[2] + p.Y * matrix[3]);
