@@ -16,13 +16,14 @@ namespace cura
 /*!
  * A class for iterating over the points in one of the polygons in a \ref Polygons object
  */
-class PolygonsPointIndex
+template<typename Paths>
+class PathsPointIndex
 {
 public:
     /*!
      * The polygons into which this index is indexing.
      */
-    const Polygons* polygons; // (pointer to const polygons)
+    const Paths* polygons; // (pointer to const polygons)
 
     unsigned int poly_idx; //!< The index of the polygon in \ref PolygonsPointIndex::polygons
 
@@ -35,7 +36,7 @@ public:
      * needed. Since the `polygons` field is const you can't ever make this
      * initialisation useful.
      */
-    PolygonsPointIndex()
+    PathsPointIndex()
     : polygons(nullptr)
     , poly_idx(0)
     , point_idx(0)
@@ -48,7 +49,7 @@ public:
      * \param poly_idx The index of the sub-polygon to point to.
      * \param point_idx The index of the vertex in the sub-polygon.
      */
-    PolygonsPointIndex(const Polygons* polygons, unsigned int poly_idx, unsigned int point_idx)
+    PathsPointIndex(const Paths* polygons, unsigned int poly_idx, unsigned int point_idx)
     : polygons(polygons)
     , poly_idx(poly_idx)
     , point_idx(point_idx)
@@ -58,7 +59,7 @@ public:
     /*!
      * Copy constructor to copy these indices.
      */
-    PolygonsPointIndex(const PolygonsPointIndex& original) = default;
+    PathsPointIndex(const PathsPointIndex& original) = default;
 
     Point p() const
     {
@@ -66,36 +67,42 @@ public:
         {
             return Point(0, 0);
         }
-        return (*polygons)[poly_idx][point_idx];
+        return make_point((*polygons)[poly_idx][point_idx]);
     }
 
     /*!
      * \brief Returns whether this point is initialised.
      */
-    bool initialized() const;
+    bool initialized() const
+    {
+        return polygons;
+    }
+
 
     /*!
      * Get the polygon to which this PolygonsPointIndex refers
      */
-    ConstPolygonRef getPolygon() const
-    {
-        return (*polygons)[poly_idx];
-    }
+    ConstPolygonRef getPolygon() const;
+
     /*!
      * Test whether two iterators refer to the same polygon in the same polygon list.
      * 
      * \param other The PolygonsPointIndex to test for equality
      * \return Wether the right argument refers to the same polygon in the same ListPolygon as the left argument.
      */
-    bool operator==(const PolygonsPointIndex& other) const
+    bool operator==(const PathsPointIndex& other) const
     {
         return polygons == other.polygons && poly_idx == other.poly_idx && point_idx == other.point_idx;
     }
-    bool operator!=(const PolygonsPointIndex& other) const
+    bool operator!=(const PathsPointIndex& other) const
     {
         return !(*this == other);
     }
-    PolygonsPointIndex& operator=(const PolygonsPointIndex& other)
+    bool operator<(const PathsPointIndex& other) const
+    {
+        return this->p() < other.p();
+    }
+    PathsPointIndex& operator=(const PathsPointIndex& other)
     {
         polygons = other.polygons;
         poly_idx = other.poly_idx;
@@ -103,13 +110,13 @@ public:
         return *this;
     }
     //! move the iterator forward (and wrap around at the end)
-    PolygonsPointIndex& operator++() 
+    PathsPointIndex& operator++() 
     { 
         point_idx = (point_idx + 1) % (*polygons)[poly_idx].size();
         return *this; 
     }
     //! move the iterator backward (and wrap around at the beginning)
-    PolygonsPointIndex& operator--() 
+    PathsPointIndex& operator--() 
     { 
         if (point_idx == 0)
         {
@@ -119,21 +126,54 @@ public:
         return *this; 
     }
     //! move the iterator forward (and wrap around at the end)
-    PolygonsPointIndex next() const 
+    PathsPointIndex next() const 
     {
-        PolygonsPointIndex ret(*this);
+        PathsPointIndex ret(*this);
         ++ret;
         return ret;
     }
     //! move the iterator backward (and wrap around at the beginning)
-    PolygonsPointIndex prev() const 
+    PathsPointIndex prev() const 
     {
-        PolygonsPointIndex ret(*this);
+        PathsPointIndex ret(*this);
         --ret;
         return ret;
     }
 };
 
+using PolygonsPointIndex = PathsPointIndex<Polygons>;
+
+
+/*!
+ * Locator to extract a line segment out of a \ref PolygonsPointIndex
+ */
+struct PolygonsPointIndexSegmentLocator
+{
+    std::pair<Point, Point> operator()(const PolygonsPointIndex& val) const
+    {
+        ConstPolygonRef poly = (*val.polygons)[val.poly_idx];
+        Point start = poly[val.point_idx];
+        unsigned int next_point_idx = (val.point_idx + 1) % poly.size();
+        Point end = poly[next_point_idx];
+        return std::pair<Point, Point>(start, end);
+    }
+};
+
+
+
+/*!
+ * Locator of a \ref PolygonsPointIndex
+ */
+template<typename Paths>
+struct PathsPointIndexLocator
+{
+    Point operator()(const PathsPointIndex<Paths>& val) const
+    {
+        return make_point(val.p());
+    }
+};
+
+using PolygonsPointIndexLocator = PathsPointIndexLocator<Polygons>;
 
 }//namespace cura
 
