@@ -1,5 +1,5 @@
 //Copyright (c) 2016 Scott Lenser
-//Copyright (c) 2018 Ultimaker B.V.
+//Copyright (c) 2020 Ultimaker B.V.
 //CuraEngine is released under the terms of the AGPLv3 or higher.
 
 #ifndef UTILS_SPARSE_POINT_GRID_H
@@ -42,6 +42,16 @@ public:
      */
     void insert(const Elem &elem);
 
+    /*!
+     * Get just any element that's within a certain radius of a point.
+     *
+     * Rather than giving a vector of nearby elements, this function just gives
+     * a single element, any element, in no particular order.
+     * \param query_pt The point to query for an object nearby.
+     * \param radius The radius of what is considered "nearby".
+     */
+    const ElemT* getAnyNearby(const Point& query_pt, coord_t radius);
+
 protected:
     using GridPoint = typename SparseGrid<ElemT>::GridPoint;
 
@@ -69,6 +79,24 @@ void SGI_THIS::insert(const Elem &elem)
     SparseGrid<ElemT>::m_grid.emplace(grid_loc,elem);
 }
 
+SGI_TEMPLATE
+const ElemT* SGI_THIS::getAnyNearby(const Point& query_pt, coord_t radius)
+{
+    const ElemT* ret = nullptr;
+    const std::function<bool (const ElemT&)>& process_func = [&ret, query_pt, radius, this](const ElemT& maybe_nearby)
+        {
+            if (shorterThen(m_locator(maybe_nearby) - query_pt, radius))
+            {
+                ret = &maybe_nearby;
+                return false;
+            }
+            return true;
+        };
+    SparseGrid<ElemT>::processNearby(query_pt, radius, process_func);
+
+    return ret;
+}
+
 
 #undef SGI_TEMPLATE
 #undef SGI_THIS
