@@ -7,7 +7,7 @@
 #include <functional> // function
 #include <limits>
 #include <optional>
-#include <vector>
+#include <memory> // unique_ptr
 
 #include "polygon.h"
 #include "SparsePointGridInclusive.h"
@@ -86,21 +86,6 @@ struct GivenDistPoint
     int pos; //!< Index to the first point in the polygon of the line segment on which the result was found
 };
 
-/*!
- * Locator to extract a line segment out of a \ref PolygonsPointIndex
- */
-struct PolygonsPointIndexSegmentLocator
-{
-    std::pair<Point, Point> operator()(const PolygonsPointIndex& val) const
-    {
-        ConstPolygonRef poly = (*val.polygons)[val.poly_idx];
-        Point start = poly[val.point_idx];
-        unsigned int next_point_idx = (val.point_idx + 1) % poly.size();
-        Point end = poly[next_point_idx];
-        return std::pair<Point, Point>(start, end);
-    }
-};
-
 typedef SparseLineGrid<PolygonsPointIndex, PolygonsPointIndexSegmentLocator> LocToLineGrid;
 
 class PolygonUtils 
@@ -337,33 +322,6 @@ public:
      */
     static ClosestPolygonPoint ensureInsideOrOutside(const Polygons& polygons, Point& from, const ClosestPolygonPoint& closest_polygon_point, int preferred_dist_inside, const Polygons* loc_to_line_polygons = nullptr, const LocToLineGrid* loc_to_line_grid = nullptr, const std::function<int(Point)>& penalty_function = no_penalty_function);
 
-    /*!
-     * Find a connecting line segment from one polygon to a collection of other polygons.
-     * 
-     * This implementation uses a sparse grid to get to an accurate result quickly
-     * 
-     * The first connection larger than \p min_connection_length and smaller than \p max_connection_length is returned.
-     * 
-     * \param poly1 The polygon in which to search for a conection
-     * \param polys2 The polygons to which to connect
-     * \param min_connection_length The minimal conection length a connection needs to have in order to stop looking for other connections
-     * \param max_connection_length The largest length of a connection to be found
-     */
-    static std::pair<ClosestPolygonPoint, ClosestPolygonPoint> findConnection(ConstPolygonRef poly1, Polygons& polys2, coord_t min_connection_length, coord_t max_connection_length, std::function<bool (std::pair<ClosestPolygonPoint, ClosestPolygonPoint>)> precondition);
-
-    /*!
-    * Find the two points in two polygons with the smallest distance.
-    * 
-    * The final connection will be close to the center of mass of the first polygon.
-    * 
-    * \warning The ClosestPolygonPoint::poly fields output parameters should be initialized with the polygons for which to find the smallest connection.
-    * 
-    * \param poly1_result Output parameter: the point at the one end of the smallest connection between its poly and \p poly2_result.poly.
-    * \param poly2_result Output parameter: the point at the other end of the smallest connection between its poly and \p poly1_result.poly.
-    * \param sample_size The number of points on each polygon to start the hill climbing search from. Use negative values for checking all combinations of points.
-    */
-    static void findSmallestConnection(ClosestPolygonPoint& poly1_result, ClosestPolygonPoint& poly2_result);
-
     /*!
     * 
     * \warning Assumes \p poly1_result and \p poly2_result have their pos and poly fields initialized!
@@ -434,7 +392,7 @@ public:
      * \param square_size The cell size used to bundle line segments (also used to chop up lines so that multiple cells contain the same long line)
      * \return A bucket grid mapping spatial locations to poly-point indices into \p polygons
      */
-    static LocToLineGrid* createLocToLineGrid(const Polygons& polygons, int square_size);
+    static std::unique_ptr<LocToLineGrid> createLocToLineGrid(const Polygons& polygons, int square_size);
 
     /*!
      * Find the line segment closest to a given point \p from within a cell-block of a size defined in the SparsePointGridInclusive \p loc_to_line
@@ -630,6 +588,24 @@ public:
      */
     static double relativeHammingDistance(const Polygons& poly_a, const Polygons& poly_b);
 
+    /*!
+     * Create an approximation of a circle.
+     *
+     * This creates a regular polygon that is supposed to approximate a circle.
+     * \param mid The center of the circle.
+     * \param radius The radius of the circle.
+     * \param a_step The angle between segments of the circle.
+     * \return A new Polygon containing the circle.
+     */
+    static Polygon makeCircle(const Point mid, const coord_t radius, const AngleRadians a_step = M_PI / 8);
+
+    /*!
+     * Connect all polygons to their holes using zero widths hole channels, so that the polygons and their outlines are connected together
+     */
+    static Polygons connect(const Polygons& input);
+
+    static void fixSelfIntersections(const coord_t epsilon, Polygons& thiss);
+
 private:
     /*!
      * Helper function for PolygonUtils::moveInside2: moves a point \p from which was moved onto \p closest_polygon_point towards inside/outside when it's not already inside/outside by enough distance.
